{*******************************************************************************
 *                                Knopf Version 1.1                            *
 *                (c) 2002 Ulrich Borghoff    Gymnasium Wilnsdorf              *
 *                           eMail: sum-delphi@web.de                          *
 * *****************************************************************************
 * Diese Komponenten Version drfte unter Delphi ab Version ab 4 laufen        *
 *******************************************************************************
 * Bitte ndern Sie diese Datei nicht ab, sondern teilen den Entwicklern Ihre  *
 * Verbesserungsvorschlge oder Fehler (und ggf. deren Berichtigung) mit.      *
 *                                                                             *
 * Danke fr Ihre Mitarbeit.                                                   *
 *******************************************************************************}

unit mKnopf;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, mSuM;

type
  TKnopf = class(tButtonControl)
   private

    FFokusErhalten, FFokusVerloren, FGeklickt: tNotifyEvent;
    FDefault: Boolean;
    FCancel: Boolean;
    FActive: Boolean;
    FModalResult: TModalResult;
    procedure SetDefault(Value: Boolean);
    procedure CMDialogKey(var Message: TCMDialogKey); message CM_DIALOGKEY;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMFocusChanged(var Message: TCMFocusChanged); message CM_FOCUSCHANGED;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure SetKnopfStyle(ADefault: Boolean); virtual;

    constructor Create(AOwner: TComponent); override;
    procedure Click; override;
    function UseRightToLeftAlignment: Boolean; override;

    property Action;
    property Anchors;
    property BiDiMode;
    property Cancel: Boolean read FCancel write FCancel default False;
    property Caption;
    property Constraints;
    property Default: Boolean read FDefault write SetDefault default False;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property ModalResult: TModalResult read FModalResult write FModalResult default 0;
    property ParentBiDiMode;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;

    procedure setzeZustand(pAktiv:Wahrheitswert);
    procedure setzeSichtbarkeit(pSichtbar:Wahrheitswert);
    procedure setzeText(pInhalt:Zeichenkette);

  protected
    procedure doEnter; override;
    procedure doExit; override;

  public

    constructor init(pLinks,pOben, pBreite, pHoehe: Zahl);  overload;
    constructor init(pLinks,pOben, pBreite, pHoehe: Zahl; pInhalt:Zeichenkette);  overload;

    procedure setzePosition(pLinks, pOben : Zahl); virtual;
    procedure setzeGroesse(pBreite, pHoehe : Zahl); virtual;

    function links:Zahl;  virtual;
    function oben:Zahl; virtual;
    function breite:Zahl;  virtual;
    function hoehe:Zahl; virtual;

    procedure verstecke; virtual;
    procedure zeige; virtual;
    function IstSichtbar:Wahrheitswert; virtual;

    procedure aktiviere; virtual;
    procedure deaktiviere; virtual;
    function istAktiviert: Wahrheitswert; virtual;

    procedure setzeFokus; virtual;
    function besitztFokus: Wahrheitswert; virtual;

    procedure setzeInhalt(pInhalt:Zeichenkette); overload; virtual;
    procedure setzeInhalt(pInhalt:Zahl); overload; virtual;

    function InhaltIstZeichenkette:Wahrheitswert; virtual;
    function InhaltIstZahl:Wahrheitswert; virtual;
    function InhaltIstGanzeZahl:Wahrheitswert; virtual;

    function inhaltAlsText:Zeichenkette; virtual;

    function inhaltAlsZahl:Zahl; virtual;
    function inhaltAlsGanzeZahl:GanzeZahl; virtual;

    procedure setzeSchriftart(pArt:Zeichenkette);
    procedure setzeSchriftGroesse(pGroesse:GanzeZahl);
    procedure setzeSchriftFarbe(pFarbe:GanzeZahl);
    procedure setzeSchriftStil(pStil:GanzeZahl);

    procedure gibFrei; virtual;

    published

    property _Inhalt: Zeichenkette read inhaltAlsText
                             write setzeText;
    property _IstAktiviert : Wahrheitswert read istAktiviert write SetzeZustand;
    property _IstSichtbar : Wahrheitswert read IstSichtbar write SetzeSichtbarkeit;
    property font;

    property Geklickt : tNotifyEvent read FGeklickt write FGeklickt;
    property FokusErhalten : tNotifyEvent read FFokusErhalten write FFokusErhalten;
    property FokusVerloren : tNotifyEvent read FFokusVerloren write FFokusVerloren;

  end;

procedure Register;

implementation

constructor tKnopf.init(pLinks,pOben, pBreite, pHoehe: Zahl);
begin
  inherited create(screen.activeForm);
  ControlStyle := [csSetCaption, csOpaque, csDoubleClicks];
  TabStop := True;
  self.setzePosition(pLinks,pOben);
  self.setzeGroesse(pBreite,pHoehe);
  self.setzeText('');
  self.parent:=screen.activeForm;
end;

constructor tKnopf.init(pLinks,pOben, pBreite, pHoehe: Zahl; pInhalt:Zeichenkette);
begin
  self.init(pLinks,pOben, pBreite, pHoehe);
  self.setzeInhalt(pInhalt);
end;

procedure tKnopf.setzePosition(pLinks, pOben:Zahl);
begin
 self.left:=round(pLinks);
 self.Top:=round(pOben);
end;

procedure tKnopf.setzeGroesse(pBreite, pHoehe:Zahl);
begin
 self.Width:=round(pBreite);
 self.Height:=round(pHoehe);
end;

function tKnopf.links:Zahl;
begin
 result:=self.left
end;

function tKnopf.oben:Zahl;
begin
  result:=self.top;
end;

function tKnopf.breite:Zahl;
begin
  result:=self.width;
end;

function tKnopf.hoehe:Zahl;
begin
  result:=self.height;
end;

procedure tKnopf.verstecke;
begin
 self.visible:=false;
end;

procedure tKnopf.zeige;
begin
 self.visible:=true;
end;

function tKnopf.IstSichtbar:Wahrheitswert;
begin
  result:=self.visible;
end;

procedure tKnopf.aktiviere;
begin
 self.enabled:=true;
end;

procedure tKnopf.deaktiviere;
begin
 self.enabled:=false;
end;

function tKnopf.istAktiviert: Wahrheitswert;
begin
 result:=self.enabled;
end;

procedure tKnopf.setzeFokus;
begin
 screen.ActiveForm.FocusControl(self);
end;

function tKnopf.besitztFokus: Wahrheitswert;
begin
 result:=self.focused;
end;

procedure tKnopf.setzeText(pInhalt:Zeichenkette);
begin
 self.caption:=pInhalt;
end;

procedure tKnopf.setzeInhalt(pInhalt:Zeichenkette);
begin
 self.caption:=pInhalt;
end;

procedure tKnopf.setzeInhalt(pInhalt:Zahl);
begin
  self.setzeText(floatToStr(pInhalt));
end;

function tKnopf.InhaltIstZeichenkette:Wahrheitswert;
begin
  result:=true;
end;

function tKnopf.InhaltIstZahl:Wahrheitswert;
  var lWert: Zahl;
      lcode : integer;
begin
  val(self.inhaltAlsText,lWert,lCode);
  result:=(lcode=0);
end;

function tKnopf.InhaltIstGanzeZahl:Wahrheitswert;
  var lWert: GanzeZahl;
      code : integer;
begin
  val(self.inhaltAlsText,lWert,Code);
  result:=(code=0);
end;

function tKnopf.InhaltAlsText:Zeichenkette;
begin
 result := self.caption;
end;

function tKnopf.inhaltAlsZahl:Zahl;
begin
  if self.InhaltistZahl then result:=strTofloat(self.inhaltAlsText)
  else
  begin
    MessageDlg('Eingabe keine Zahl.',mtError,[mbOK],0);
    result:=0;
  end;
end;

function tKnopf.inhaltAlsGanzeZahl:GanzeZahl;
begin
  if self.InhaltIstGanzeZahl then result:=strToInt(self.inhaltAlsText)
  else
  begin
    MessageDlg('Eingabe keine ganze Zahl.',mtError,[mbOK],0);
    result:=0;
  end;
end;

procedure tKnopf.setzeSchriftart(pArt:Zeichenkette);
begin
  self.Font.Name:=pArt;
end;

procedure tKnopf.setzeSchriftGroesse(pGroesse:GanzeZahl);
begin
  self.Font.size:=pGroesse;
end;

procedure tKnopf.setzeSchriftFarbe(pFarbe:GanzeZahl);
begin
  self.Font.Color:=SuMfarbumrechnung(pFarbe); {farbumrechnung aus mSuM}
end;

procedure tKnopf.setzeSchriftStil(pStil:GanzeZahl);
begin
  self.Font.Style:=SuMstilumrechnung(pStil);  {stilumrechnung aus mSuM}
end;

procedure tKnopf.gibFrei;
begin
 self.destroy;
end;



// private

procedure tKnopf.setzeZustand(pAktiv:Wahrheitswert);
begin
 self.enabled:=pAktiv;
end;

procedure tKnopf.setzeSichtbarkeit(pSichtbar:Wahrheitswert);
begin
 self.visible:=pSichtbar;
end;

// Ereignisse

procedure tKnopf.doEnter;
begin
  if assigned(FokusErhalten) then FokusErhalten(self);
  inherited doEnter;
end;

procedure tKnopf.doExit;
begin
  if assigned(FokusVerloren) then FokusVerloren(self);
  inherited doExit;
end;

procedure tKnopf.Click;
var
  Form: TCustomForm;
begin
  Form := GetParentForm(Self);
  if Form <> nil then Form.ModalResult := ModalResult;
    if Assigned(FGeklickt) then FGeklickt(Self);
  inherited Click;
end;





// Registrierung

procedure Register;
begin
  RegisterComponents('Schule', [TKnopf]);
end;


{ tKnopf }

constructor tKnopf.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csSetCaption, csOpaque, csDoubleClicks];
  Width := 75;
  Height := 25;
  TabStop := True;
end;

function tKnopf.UseRightToLeftAlignment: Boolean;
begin
  Result := False;
end;

procedure tKnopf.SetKnopfStyle(ADefault: Boolean);
const
  BS_MASK = $000F;
var
  Style: Word;
begin
  if HandleAllocated then
  begin
    if ADefault then Style := BS_DEFPUSHBUTTON else Style := BS_PUSHBUTTON;
    if GetWindowLong(Handle, GWL_STYLE) and BS_MASK <> Style then
      SendMessage(Handle, BM_SETSTYLE, Style, 1);
  end;
end;

procedure tKnopf.SetDefault(Value: Boolean);
var
  Form: TCustomForm;
begin
  FDefault := Value;
  if HandleAllocated then
  begin
    Form := GetParentForm(Self);
    if Form <> nil then
      Form.Perform(CM_FOCUSCHANGED, 0, Longint(Form.ActiveControl));
  end;
end;

procedure tKnopf.CreateParams(var Params: TCreateParams);
const
  ButtonStyles: array[Boolean] of DWORD = (BS_PUSHBUTTON, BS_DEFPUSHBUTTON);
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, 'BUTTON');
  Params.Style := Params.Style or ButtonStyles[FDefault];
end;

procedure tKnopf.CreateWnd;
begin
  inherited CreateWnd;
  FActive := FDefault;
end;

procedure tKnopf.CNCommand(var Message: TWMCommand);
begin
  if Message.NotifyCode = BN_CLICKED then Click;
end;

procedure tKnopf.CMDialogKey(var Message: TCMDialogKey);
begin
  with Message do
    if  (((CharCode = VK_RETURN) and FActive) or
      ((CharCode = VK_ESCAPE) and FCancel)) and
      (KeyDataToShiftState(Message.KeyData) = []) and CanFocus then
    begin
      Click;
      Result := 1;
    end else
      inherited;
end;

procedure tKnopf.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and CanFocus then
    begin
      Click;
      Result := 1;
    end else
      inherited;
end;

procedure tKnopf.CMFocusChanged(var Message: TCMFocusChanged);
begin
  with Message do
    if Sender is tKnopf then
      FActive := Sender = Self
    else
      FActive := FDefault;
  SetKnopfStyle(FActive);
  inherited;
end;

procedure tKnopf.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  DefaultHandler(Message);
end;

end.
