{*******************************************************************************
 *                              Textfeld Version 1.1.1                         *
 *                (c) 2002 Ulrich Borghoff    Gymnasium Wilnsdorf              *
 *                           eMail: sum-delphi@web.de                          *
 * *****************************************************************************
 * Diese Komponenten Version drfte unter Delphi ab Version ab 4 laufen        *
 *******************************************************************************
 * Bitte ndern Sie diese Datei nicht ab, sondern teilen den Entwicklern Ihre  *
 * Verbesserungsvorschlge oder Fehler (und ggf. deren Berichtigung) mit.      *
 *                                                                             *
 * Danke fr Ihre Mitarbeit.                                                   *
 *******************************************************************************}


unit mTextfeld;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, mSuM;

type
  TTextfeld = class(TCustomEdit)

 private
    FFokusErhalten, FFokusVerloren, FTextGeaendert, FEingabeBestaetigt: tNotifyEvent;

    procedure setzeZustand(pAktiv:Wahrheitswert);
    procedure setzeSichtbarkeit(pSichtbar:Wahrheitswert);
    procedure setzeText(pInhalt:Zeichenkette);

  protected
    procedure doEnter; override;
    procedure doExit; override;
    procedure change; override;
    procedure KeyPress(var Key: Char); override;

  public

    constructor Create(AOwner: TComponent); override;
    constructor init(pLinks,pOben, pBreite, pHoehe: Zahl);  overload;
    constructor init(pLinks,pOben, pBreite, pHoehe: Zahl; pInhalt:Zeichenkette);  overload;

    procedure setzePosition(pLinks, pOben : Zahl); virtual;
    procedure setzeGroesse(pBreite, pHoehe : Zahl); virtual;

    function links:Zahl;  virtual;
    function oben:Zahl; virtual;
    function breite:Zahl;  virtual;
    function hoehe:Zahl; virtual;

    procedure verstecke; virtual;
    procedure zeige; virtual;
    function IstSichtbar:Wahrheitswert; virtual;

    procedure aktiviere; virtual;
    procedure deaktiviere; virtual;
    function istAktiviert: Wahrheitswert; virtual;

    procedure setzeFokus; virtual;
    function besitztFokus: Wahrheitswert; virtual;

    procedure setzeInhalt(pInhalt:Zeichenkette); overload; virtual;
    procedure setzeInhalt(pInhalt:Zahl); overload; virtual;

    function InhaltIstZeichenkette:Wahrheitswert; virtual;
    function InhaltIstZahl:Wahrheitswert; virtual;
    function InhaltIstGanzeZahl:Wahrheitswert; virtual;

    function inhaltAlsText:Zeichenkette; virtual;

    function inhaltAlsZahl:Zahl; virtual;
    function inhaltAlsGanzeZahl:GanzeZahl; virtual;

    procedure setzeSchriftart(pArt:Zeichenkette);
    procedure setzeSchriftGroesse(pGroesse:GanzeZahl);
    procedure setzeSchriftFarbe(pFarbe:GanzeZahl);
    procedure setzeSchriftStil(pStil:GanzeZahl);

    procedure gibFrei; virtual;

    published

    property _Inhalt: Zeichenkette read inhaltAlsText
                             write setzeText;
    property _IstAktiviert : Wahrheitswert read istAktiviert write SetzeZustand;
    property _IstSichtbar : Wahrheitswert read IstSichtbar write SetzeSichtbarkeit;
    property font;
    property FokusErhalten : tNotifyEvent read FFokusErhalten write FFokusErhalten;
    property FokusVerloren : tNotifyEvent read FFokusVerloren write FFokusVerloren;
    property TextGeaendert : tNotifyEvent read FTextGeaendert write FTextGeaendert;
    property EingabeBestaetigt : tNotifyEvent read FEingabeBestaetigt write FEingabeBestaetigt;

  end;

procedure Register;

implementation

constructor tTextfeld.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  self.setzeInhalt('');
  DecimalSeparator:='.'
end;

constructor tTextfeld.init(pLinks,pOben, pBreite, pHoehe: Zahl);
begin
  inherited create(screen.activeForm);
  ControlStyle := [csSetCaption, csOpaque, csDoubleClicks];
  TabStop := True;
  self.setzePosition(pLinks,pOben);
  self.setzeGroesse(pBreite,pHoehe);
  self.setzeText('');
  self.parent:=screen.activeForm;
end;

constructor tTextfeld.init(pLinks,pOben, pBreite, pHoehe: Zahl; pInhalt:Zeichenkette);
begin
  self.init(pLinks,pOben, pBreite, pHoehe);
  self.setzeInhalt(pInhalt);
end;

procedure tTextfeld.setzePosition(pLinks, pOben:Zahl);
begin
 self.left:=round(pLinks);
 self.Top:=round(pOben);
end;

procedure tTextfeld.setzeGroesse(pBreite, pHoehe:Zahl);
begin
 self.Width:=round(pBreite);
 self.Height:=round(pHoehe);
end;

function tTextfeld.links:Zahl;
begin
 result:=self.left
end;

function tTextfeld.oben:Zahl;
begin
  result:=self.top;
end;

function tTextfeld.breite:Zahl;
begin
  result:=self.width;
end;

function tTextfeld.hoehe:Zahl;
begin
  result:=self.height;
end;

procedure tTextfeld.verstecke;
begin
 self.visible:=false;
end;

procedure tTextfeld.zeige;
begin
 self.visible:=true;
end;

function tTextfeld.IstSichtbar:Wahrheitswert;
begin
  result:=self.visible;
end;

procedure tTextfeld.aktiviere;
begin
 self.enabled:=true;
end;

procedure tTextfeld.deaktiviere;
begin
 self.enabled:=false;
end;

function tTextfeld.istAktiviert: Wahrheitswert;
begin
 result:=self.enabled;
end;

procedure tTextfeld.setzeFokus;
begin
 screen.ActiveForm.FocusControl(self);
 self.AutoSelect:=true;
 self.SelectAll;
end;

function tTextfeld.besitztFokus: Wahrheitswert;
begin
 result:=self.focused;
end;

procedure tTextfeld.setzeText(pInhalt:Zeichenkette);
begin
 self.text:=pInhalt;
end;

procedure tTextfeld.setzeInhalt(pInhalt:Zeichenkette);
begin
 self.text:=pInhalt;
end;

procedure tTextfeld.setzeInhalt(pInhalt:Zahl);
begin
  self.setzeText(floatToStr(pInhalt));
end;

function tTextfeld.InhaltIstZeichenkette:Wahrheitswert;
begin
  result:=true;
end;

function tTextfeld.InhaltIstZahl:Wahrheitswert;
  var lWert: Zahl;
      lcode : integer;
begin
  val(self.inhaltAlsText,lWert,lCode);
  result:=(lcode=0);
end;

function tTextfeld.InhaltIstGanzeZahl:Wahrheitswert;
  var lWert: GanzeZahl;
      code : integer;
begin
  val(self.inhaltAlsText,lWert,Code);
  result:=(code=0);
end;

function tTextfeld.InhaltAlsText:Zeichenkette;
begin
 result := self.Text;
end;

function tTextfeld.inhaltAlsZahl:Zahl;
begin
  if self.InhaltistZahl then result:=strTofloat(self.inhaltAlsText)
  else
  begin
    MessageDlg('Eingabe keine Zahl.',mtError,[mbOK],0);
    result:=0;
  end;
end;

function tTextfeld.inhaltAlsGanzeZahl:GanzeZahl;
begin
  if self.InhaltIstGanzeZahl then result:=strToInt(self.inhaltAlsText)
  else
  begin
    MessageDlg('Eingabe keine ganze Zahl.',mtError,[mbOK],0);
    result:=0;
  end;
end;

procedure tTextfeld.setzeSchriftart(pArt:Zeichenkette);
begin
  self.Font.Name:=pArt;
end;

procedure tTextfeld.setzeSchriftGroesse(pGroesse:GanzeZahl);
begin
  self.Font.size:=pGroesse;
end;

procedure tTextfeld.setzeSchriftFarbe(pFarbe:GanzeZahl);
begin
  self.Font.Color:=SuMfarbumrechnung(pFarbe); {farbumrechnung aus mSuM}
end;

procedure tTextfeld.setzeSchriftStil(pStil:GanzeZahl);
begin
  self.Font.Style:=SuMstilumrechnung(pStil);  {stilumrechnung aus mSuM}
end;


procedure tTextfeld.gibFrei;
begin
 self.destroy;
end;

// private

procedure tTextfeld.setzeZustand(pAktiv:Wahrheitswert);
begin
 self.enabled:=pAktiv;
end;

procedure tTextfeld.SetzeSichtbarkeit(pSichtbar:Wahrheitswert);
begin
 self.visible:=pSichtbar;
end;

// Ereignisse

procedure tTextfeld.doEnter;
begin
  if assigned(FokusErhalten) then FokusErhalten(self);
  inherited doEnter;
end;

procedure tTextfeld.doExit;
begin
  if assigned(FokusVerloren) then FokusVerloren(self);
  inherited doExit;
end;

procedure tTextfeld.change;
begin
  if assigned(TextGeaendert) then TextGeaendert(self);
  inherited change;
end;

procedure tTextfeld.KeyPress(var Key: Char);
begin
  if assigned(EingabeBestaetigt) and (key=#13) then EingabeBestaetigt(self);
  inherited KeyPress(key);
end;

// Registrierung

procedure Register;
begin
  RegisterComponents('Schule', [tTextfeld]);
end;


end.
